///////////////////////////////////////////////////////////
//  SystemFactory.cpp
//  Implementation of the Class SystemFactory
//  Created on:      29-4-2009 15:41:19
//  Original author: Ueno
///////////////////////////////////////////////////////////

#include "Application/EELTBootWriter/SystemFactory.h"

using Application::EELTBootWriter::SystemFactory;
#include "Unit/CPUBoard/CPUBoardFactory.h"
#include "Unit/CPUBoard/CPUBoard.h"
#include <new>
#include "OsWrapper/OSFactory.h"
#include "Common/TypeDef/UFourByte.h"
#include "Common/TypeDef/UOneByte.h"
#include "Device/CPU/H8S_2000/H8S_2000_REGS.h" 		// for DEBUG
#include "Common/BasicType/util.h"
#include "Device/CPU/SCI.h"
#include "Device/CPU/CPU.h"
#include "UI/CommandInterpreter/Command.h"
#include "UI/CommandInterpreter/CommandInterpreterFactory.h"
#include "Unit/Communication/CommunicationFactory.h"
#include "WriteRom.h"
#include "EraseRom.h"
#include "DumpMemory.h"
#include "ExceptionCommand.h"

SystemFactory::SystemFactory(){}
SystemFactory::~SystemFactory(){}

void SystemFactory::startup(){
	Unit::CPUBoard::CPUBoard* pCPUBoard;
	Device::CPU::SCI* pSCI;
	bool bRet=true;

	//=============== CPU ================
	{
		Unit::CPUBoard::CPUBoardFactory* pCpuBoardFactory;
		pCpuBoardFactory=Unit::CPUBoard::CPUBoardFactory::getInstance();
		bRet=pCpuBoardFactory->create(Unit::CPUBoard::CPUBoard::CBT_EE_LT_BOOT_WRITER2,Unit::CPUBoard::CPUBoard::BM_APPLI);
		if(bRet==true)
		{ pCPUBoard=pCpuBoardFactory->getIF(); }
	}
	if(bRet==false) goto ERROR;
		
	//=============== SCI ================
	{
		Device::CPU::SCI::CommunicationParam cp;
		Device::CPU::CPU* pCPU;
	
		cp.cm=Device::CPU::SCI::CM_SYNCHRONUS_TOTNOAL;
		cp.bps=38400;
		cp.dl=Device::CPU::SCI::DL_8BIT;
		cp.sb=Device::CPU::SCI::SB_1;
		cp.parity=Device::CPU::SCI::P_NONE;
		cp.mp=Device::CPU::SCI::MP_DISABLE;
		pCPU=pCPUBoard->getCPU();
		bRet=pCPU->addSCI(Device::CPU::SCI::TI_H8_SCI1,cp,&pSCI);
	}
	if(bRet==false) goto ERROR;
#if defined(APPL_DEBUG)
	rawDisp2Digit(P2DDR_ADDR,P2DR_ADDR,(unsigned long)0x01);
#endif
	// ============== OSWrapperν ===================
	{
		OsWrapper::OSFactory* pOsFactory;
		pOsFactory=OsWrapper::OSFactory::getInstance();
		pOsFactory->setTimerCycle(50UL*128U*256U);	// OSޡ1.6msʬǽ
		bRet=pOsFactory->init(pCPUBoard);
	}
	if(bRet==false) goto ERROR;

#if defined(APPL_DEBUG)
	rawDisp2Digit(P2DDR_ADDR,P2DR_ADDR,(unsigned long)0x02);
#endif
#if defined(SCI_DEBUG_PERFORMANCE)
	{
		volatile unsigned char * pp;	
		pp=P2DDR_ADDR;
		*pp |=0xff;
		pp=P2DR_ADDR;
		*pp =0;
	}
#endif
#if defined(SCI_DEBUG_PERFORMANCE1)
	{
		volatile unsigned char * pp;	
		pp=P2DDR_ADDR;
		*pp |=0xff;
		pp=P2DR_ADDR;
		*pp =0;
	}
#endif

	// ============== Communicationν ===================
	{
		UI::CommandInterpreter::CommandInterpreterFactory* pCIFactory;
		UI::CommandInterpreter::CommandInterpreter* pCI;
		UI::CommandInterpreter::Command* pCommand;
		Unit::Communication::CommunicationFactory* pCFactory;
		Unit::Communication::Com* pCom;
	
		pCFactory = new Unit::Communication::CommunicationFactory();
		if(pCFactory==NULL) {bRet=false; goto COM_INIT_ERROR;}
#if defined(APPL_DEBUG)
	rawDisp2Digit(P2DDR_ADDR,P2DR_ADDR,(unsigned long)0x03);
#endif
		if(pCFactory->init(pSCI,&pCom)==false) {bRet=false; goto COM_INIT_ERROR;}
		delete pCFactory;
#if defined(APPL_DEBUG)
	rawDisp2Digit(P2DDR_ADDR,P2DR_ADDR,(unsigned long)0x04);
#endif
		// ============== CommandInterpreterν ===================
		pCommand=new ExceptionCommand();
		if(pCommand==NULL) {bRet=false; goto COM_INIT_ERROR;}

		pCIFactory=new UI::CommandInterpreter::CommandInterpreterFactory(pCom,pCommand);
		if(pCIFactory==NULL) {bRet=false; goto COM_INIT_ERROR;}
	
#if defined(APPL_DEBUG)
	rawDisp2Digit(P2DDR_ADDR,P2DR_ADDR,(unsigned long)0x05);
#endif
		pCI=pCIFactory->getCommandInterpreter();
		if(pCI==NULL) {bRet=false; goto COM_INIT_ERROR;}

#if defined(APPL_DEBUG)
	rawDisp2Digit(P2DDR_ADDR,P2DR_ADDR,(unsigned long)0x06);
#endif
		pCommand=new WriteRom(pCPUBoard);
		if(pCommand==NULL) {bRet=false; goto COM_INIT_ERROR;}
		pCI->addCommand(pCommand);
	
		pCommand=new EraseRom(pCPUBoard);
		if(pCommand==NULL) {bRet=false; goto COM_INIT_ERROR;}
		pCI->addCommand(pCommand);
	
		pCommand=new DumpMemory();
		if(pCommand==NULL) {bRet=false; goto COM_INIT_ERROR;}
		pCI->addCommand(pCommand);
/*
		pCommand=new ModifyMemory();
		if(pCommand==NULL) {bRet=false; goto COM_INIT_ERROR;}
		pCI->addCommand(pCommand);
*/

#if defined(APPL_DEBUG)
	rawDisp2Digit(P2DDR_ADDR,P2DR_ADDR,(unsigned long)0x07);
#endif
		pCI->setPrompt("BW>");
	
		pCIFactory->init("EE_LT BootWriter2 V1.0");
#if defined(APPL_DEBUG)
	rawDisp2Digit(P2DDR_ADDR,P2DR_ADDR,(unsigned long)0x08);
#endif
		delete pCIFactory;
COM_INIT_ERROR:;
	}
	if(bRet==false) goto ERROR;

#if defined(APPL_DEBUG)
	rawDisp2Digit(P2DDR_ADDR,P2DR_ADDR,(unsigned long)0x09);
#endif
	while(-1) {}
ERROR:
	while(-1) {}
}
